package com.ruoyi.agentcenter.listener;

import cn.hutool.core.lang.Assert;
import com.nsgk.agentcentersdk.core.NSProtocol;
import com.nsgk.agentcentersdk.entity.NSContractionEntity;
import com.ruoyi.agentcenter.domain.TAgentContraction;
import com.ruoyi.agentcenter.domain.TAgentTask;
import com.ruoyi.agentcenter.object.ContractionSession;
import com.ruoyi.agentcenter.object.Message;
import com.ruoyi.agentcenter.object.Result;
import com.ruoyi.agentcenter.service.ITAgentContractionService;
import com.ruoyi.agentcenter.service.ITAgentTaskService;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.handler.CallbackHandler;
import com.ruoyi.common.utils.handler.HandlerEngine;
import com.ruoyi.common.utils.sql.SqlUtil;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.Date;

@Component
public class ContractionListener
{
    @Autowired
    private ITAgentContractionService itAgentContractionService;
    @Autowired
    private ITAgentTaskService itAgentTaskService;
    @Autowired
    private ISysDeptService    iSysDeptService;

    @PostConstruct
    public void init()
    {
        HandlerEngine.Register(this);
    }

    @CallbackHandler(protocol = "" + NSProtocol.NS_PROTOCOL_CONTRACTION)
    public void handle(ContractionSession session)
    {
        TAgentContraction contraction = conv(session.message);
        Result result = session.result;

        SqlUtil.transaction(() -> {
            TAgentTask task = syncTask(session.message.getData(), contraction);
            contraction.setTaskId(task.getId());
            itAgentContractionService.insertTAgentContraction(contraction);
        });

        result.setData(contraction);
    }

    private TAgentContraction conv(Message<NSContractionEntity> message)
    {
        TAgentContraction contraction = new TAgentContraction();
        NSContractionEntity data = message.getData();
        contraction.setBookId(data.getBookId())
                .setDeptId(data.getDeptId())
                .setOutId(data.getOutId())
                .setOrgCode(data.getOrgCode())
                .setName(data.getName())
                .setBuildingTime(data.getBuildingTime())
                .setOrderAt(new Date())
                .setAgentStatus(TAgentTask.AGENT_STATUS_READY)
                .setOrderStatus("2")
                .normalized()
                ;
        contraction.setCreateBy("admin");
        return contraction;
    }

    private TAgentTask syncTask(NSContractionEntity entity, TAgentContraction contraction)
    {
        String orgCode = contraction.getOrgCode();

        TAgentTask tAgentTask = itAgentTaskService.getTAgentTask(orgCode, contraction.getOrderYear(), contraction.getOrderMonth(), (x) -> {
            x.setBookId(contraction.getBookId())
                .setBookName(entity.getBookName())
            ;
        });

        TAgentTask task = new TAgentTask();
        task.setContracCount(1);
        task.setId(tAgentTask.getId());
        itAgentTaskService.updateTAgentTaskCount(task);

        return task;
    }
}
