package com.ruoyi.agentcenter.service.impl;

import cn.hutool.core.lang.Assert;
import com.nsgk.agentcentersdk.api.NSSDKServer;
import com.nsgk.agentcentersdk.core.NSProtocol;
import com.nsgk.agentcentersdk.core.NSReportObject;
import com.nsgk.agentcentersdk.entity.NSContractionEntity;
import com.nsgk.agentcentersdk.entity.NSEntity;
import com.nsgk.agentcentersdk.err.NSErrGlobal;
import com.nsgk.agentcentersdk.err.NSErrno;
import com.nsgk.agentcentersdk.err.NSException;
import com.ruoyi.agentcenter.object.ContractionSession;
import com.ruoyi.agentcenter.object.Message;
import com.ruoyi.agentcenter.object.Result;
import com.ruoyi.agentcenter.object.Session;
import com.ruoyi.agentcenter.service.IAgentCenter;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.handler.HandlerEngine;
import com.ruoyi.system.mapper.SysDeptMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;

@Service
public class AgentCenterImpl implements IAgentCenter
{
    @Autowired
    private SysDeptMapper sysDeptMapper;

    public int getProtocol(HttpServletRequest request)
    {
        int protocol = NSSDKServer.GetProtocol(request);
        if(!NSProtocol.IsValid(protocol))
        {
            NSErrGlobal.Err(NSErrno.ERRNO_SYS_INVALID_PROTOCOL);
            throw new NSException(NSErrno.ERRNO_SYS_INVALID_PROTOCOL, String.format("未知的协议类型: 0x%x", protocol));
        }
        return protocol;
    }

    public Message<?> getMessage(NSSDKServer server, HttpServletRequest request)
    {
        NSReportObject<?> reportObject;
        Message<?> message;

        int protocol = getProtocol(request);
        Class<? extends NSEntity> clazz;
        switch(protocol)
        {
            case NSProtocol.NS_PROTOCOL_CONTRACTION:
                clazz = NSContractionEntity.class;
                break;
            default:
                return null;
        }
        reportObject = server.Recv(request, clazz);
        message = new Message<>(protocol, reportObject.getIdentifier(), reportObject.getTimestamp(), reportObject.getData());
        return message;
    }

    public Session<?, ?> createSession(Message<?> message)
    {
        Session<?, ?> session;
        int protocol;

        protocol = message.getProtocol();
        switch(protocol)
        {
            case NSProtocol.NS_PROTOCOL_CONTRACTION:
                session = new ContractionSession((Message<NSContractionEntity>) message);
                break;
            default:
                return null;
        }
        return session;
    }

    private void check(Message<?> message, HttpServletRequest request)
    {
        Assert.notNull(message, "无效消息");
        String identifier = message.getIdentifier();
        Assert.notBlank(identifier, "无效客户端标识");
        SysDept dept = sysDeptMapper.selectSysDeptByOrgCode(identifier);
        Assert.notNull(dept, "不支持的地区: {}", identifier);
        String remoteUrl = dept.getRemoteUrl();
        Assert.notBlank(remoteUrl, "该地区无项目: {}", identifier);
        /*try
        {
            URL url = new URL(remoteUrl);
            String host = url.getHost();
            String clientHost = request.getRemoteAddr();
            Assert.isTrue(clientHost.equals(host), "该客户端部署在不支持的地址: {}", clientHost);

            int port = url.getPort();
            int clientPort = request.getRemotePort();
            Assert.isTrue(clientPort == port, "该客户端部署在不支持的地址: {}", clientPort);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new RuntimeException(e);
        }*/
    }

    @Override
    public Result<?> postHandle(NSSDKServer server, HttpServletRequest request)
    {
        Message<?> message = getMessage(server, request);
        check(message, request);
        Session<?, ?> session = createSession(message);
        HandlerEngine.Post(session);
        return session.result;
    }
}
